@@ -232,7 +232,14 @@ SQLRETURN myodbc_do_connect(DBC *dbc, DataSource *ds)
 #if MYSQL_VERSION_ID >= 50610
   if (ds->can_handle_exp_pwd)
   {
-    mysql_options(mysql, MYSQL_OPT_CAN_HANDLE_EXPIRED_PASSWORDS, (char *)&on);
+    //mysql_options(mysql, MYSQL_OPT_CAN_HANDLE_EXPIRED_PASSWORDS, (char *)&on);
+  }
+#endif
+
+#if (MYSQL_VERSION_ID >= 50527 && MYSQL_VERSION_ID < 50600) || MYSQL_VERSION_ID >= 50607
+  if (ds->enable_cleartext_plugin)
+  {
+    mysql_options(mysql, MYSQL_ENABLE_CLEARTEXT_PLUGIN, (char *)&on);
   }
 #endif
 
@@ -513,7 +520,16 @@ SQLRETURN SQL_API MySQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
    This also allows us to get pszDRIVER (if not already given).
   */
   if (ds->name)
-    ds_lookup(ds);
+  {
+     ds_lookup(ds);
+    
+    /* 
+      If DSN is used:
+      1 - we want the connection string options to override DSN options
+      2 - no need to check for parsing erros as it was done before
+    */
+    ds_from_kvpair(ds, szConnStrIn, (SQLWCHAR)';');
+  }
 #endif
 
   /* If FLAG_NO_PROMPT is not set, force prompting off. */
