@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2001, 2012, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2001, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -132,6 +132,10 @@ extern "C"
 # define ER_MUST_CHANGE_PASSWORD_LOGIN 1820
 #endif
 
+#ifndef CR_AUTH_PLUGIN_CANNOT_LOAD_ERROR
+# define CR_AUTH_PLUGIN_CANNOT_LOAD_ERROR 2059
+#endif
+
 /* Connection flags to validate after the connection*/
 #define CHECK_AUTOCOMMIT_ON	1  /* AUTOCOMMIT_ON */
 #define CHECK_AUTOCOMMIT_OFF	2  /* AUTOCOMMIT_OFF */
@@ -235,6 +239,9 @@ typedef struct {
     /* connection we were allocated on */
     struct tagDBC *dbc;
   } exp;
+#ifdef THREAD
+  pthread_mutex_t lock;
+#endif
 } DESC;
 
 /* descriptor record */
@@ -325,7 +332,9 @@ typedef struct	tagENV
   SQLINTEGER   odbc_ver;
   LIST	       *connections;
   MYERROR      error;
-
+#ifdef THREAD
+  pthread_mutex_t lock;
+#endif
 } ENV;
 
 
