@@ -564,8 +564,28 @@ SQLRETURN insert_param(STMT *stmt, char **toptr, DESC* apd,
 
                 if (time->fraction)
                 {
-                  sprintf(buff + length, ".%09d", time->fraction);
-                  length+= 10;
+                  char *tmp_buf= buff + length;
+
+                  /* Start cleaning from the end */
+                  int tmp_pos= 9;
+
+                  sprintf(tmp_buf, ".%09d", time->fraction);
+
+                  /*
+                    ODBC specification defines nanoseconds granularity for
+                    the fractional part of seconds. MySQL only supports 
+                    microseconds for TIMESTAMP, TIME and DATETIME.
+
+                    We are trying to remove the trailing zeros because this 
+                    does not really modify the data, but often helps to substitute
+                    9 digits with only 6.
+                  */
+                  while (tmp_pos && tmp_buf[tmp_pos] == '0')
+                  {
+                    tmp_buf[tmp_pos--]= 0;
+                  }
+
+                  length+= tmp_pos + 1;
                 }
 
                 data= buff;
@@ -721,19 +741,35 @@ SQLRETURN insert_param(STMT *stmt, char **toptr, DESC* apd,
             {
                 char *to= buff, *from= data;
                 char *end= from+length;
+                char *local_thousands_sep= thousands_sep;
+                char *local_decimal_point= decimal_point;
+                uint local_thousands_sep_length= thousands_sep_length;
+                uint local_decimal_point_length= decimal_point_length;
+
+                if (!stmt->dbc->ds->dont_use_set_locale)
+                {
+                  /* force use of . as decimal point */
+                  local_thousands_sep= ",";
+                  local_thousands_sep_length= 1;
+                  local_decimal_point= ".";
+                  local_decimal_point_length= 1;
+                }
+
                 while ( *from && from < end )
                 {
-                  /* I wonder if following code really respects dont_use_set_locale
-                     and if it does, then how? */
-                    if ( from[0] == thousands_sep[0] && is_prefix(from,thousands_sep) )
-                        from+= thousands_sep_length;
-                    else if ( from[0] == decimal_point[0] && is_prefix(from,decimal_point) )
-                    {
-                        from+= decimal_point_length;
-                        *to++='.';
-                    }
-                    else
-                        *to++= *from++;
+                  if ( from[0] == local_thousands_sep[0] && is_prefix(from,local_thousands_sep) )
+                  {
+                    from+= local_thousands_sep_length;
+                  }
+                  else if ( from[0] == local_decimal_point[0] && is_prefix(from,local_decimal_point) )
+                  {
+                    from+= local_decimal_point_length;
+                    *to++='.';
+                  }
+                  else
+                  {
+                *to++= *from++;
+                  }
                 }
                 if ( to == buff )
                     *to++='0';    /* Fix for empty strings */
