@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -1056,6 +1056,33 @@ DECLARE_TEST(t_sqlcolumns_after_select)
 }
 
 
+/* 
+  Bug #69448 MySQL ODBC drivers incorrectly quotes TableName when 
+  calling SQLPrimaryKeys
+*/
+DECLARE_TEST(t_bug69448)
+{
+ 
+  SQLCHAR buff[255];
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS `table``69448`");
+
+  ok_sql(hstmt, "CREATE TABLE `table``69448`(id int primary key)");
+
+  ok_stmt(hstmt, SQLPrimaryKeys(hstmt, NULL, SQL_NTS, NULL, SQL_NTS, 
+                               "table`69448", SQL_NTS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /* check the table name and PK name */
+  is_str(my_fetch_str(hstmt, buff, 3), "table`69448", 11);
+  is_str(my_fetch_str(hstmt, buff, 4), "id", 2);
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS `table``69448`");
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(t_bug37621)
   ADD_TEST(t_bug34272)
@@ -1073,6 +1100,7 @@ BEGIN_TESTS
   ADD_TEST(t_bug14085211_part1)
   ADD_TODO(t_bug14085211_part2)
   ADD_TEST(t_sqlcolumns_after_select)
+  ADD_TODO(t_bug69448)
 END_TESTS
 
 myoption &= ~(1 << 30);
