@@ -53,6 +53,7 @@
 
 #include <sql.h>
 #include <sqlext.h>
+#include <odbcinst.h>
 
 /* for clock() */
 #include <time.h>
@@ -149,6 +150,8 @@ const SQLCHAR *testname_suffix="";
 
 char *SKIP_REASON= NULL;
 
+#define USE_DRIVER (char *)-1
+
 typedef int (*test_func)(SQLHDBC, SQLHSTMT, SQLHENV);
 static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
 		       const char *text, const char *file, int line);
@@ -226,6 +229,9 @@ void mem_gc_init()
   gc_blk.counter= 0;
 }
 
+#define DECLARE_BASIC_HANDLES(E, C, S) SQLHENV E; \
+  SQLHDBC C; \
+  SQLHSTMT S
 
 #define BEGIN_TESTS my_test tests[]= {
 #define ADD_TEST(name) { #name, name, OK   },
@@ -472,6 +478,21 @@ do { \
 } while (0)
 
 
+/**
+  Verify that the results of an ODBC function call on an environment handle
+  was SQL_SUCCESS or SQL_SUCCESS_WITH_INFO.
+
+  @param call    The function call
+*/
+#define ok_install(call) \
+do { \
+  BOOL rc= (call); \
+  print_diag_installer(rc, #call, __FILE__, __LINE__); \
+  if (!rc) \
+    return FAIL; \
+} while (0)
+
+
 /**
   Verify that a Boolean expression is true.
   It's recommended to use is_num, is_str, etc macros instead of "is(a==b)",
@@ -571,6 +592,8 @@ int check_sqlstate_ex(SQLHANDLE hnd, SQLSMALLINT hndtype, char *sqlstate)
 
 
 /**
+  Print error and diagnostic information for ODBC API functions that did not
+  finish with SQL_SUCCESS(_WITH_INFO) result
 */
 static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
 		       const char *text, const char *file, int line)
@@ -599,6 +622,32 @@ static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
 }
 
 
+/**
+  Print error and diagnostic information for ODBC INSTALLER API functions 
+  that did not return TRUE (1)
+*/
+static void print_diag_installer(BOOL is_success, const char *text, 
+                                 const char *file, int line)
+{
+  if (!is_success)
+  {
+    SQLCHAR     message[SQL_MAX_MESSAGE_LENGTH];
+    SQLINTEGER  error_code;
+    SQLSMALLINT length;
+    SQLRETURN   drc;
+
+    drc= SQLInstallerError(1, &error_code, message, SQL_MAX_MESSAGE_LENGTH - 1, &length);
+
+    if (SQL_SUCCEEDED(drc))
+      printf("# [%s] %s in %s on line %d\n",
+             text, message, file, line);
+    else
+      printf("# Did not get expected diagnostics from SQLInstallerError() = %d"
+             " in file %s on line %d\n", drc, file, line);
+  }
+}
+
+
 /* UTILITY MACROS */
 #define myenv(henv,r)  \
   do { \
@@ -1095,31 +1144,47 @@ int mydrvconnect(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt, SQLCHAR *connIn)
    and my_str_options, respectively.
    myoption, mysock and myport values are used. */
 int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
-                   const SQLCHAR *pwd, const SQLCHAR *options)
+                   const SQLCHAR *pwd, const SQLCHAR *db, 
+                   const SQLCHAR *options)
 {
-  SQLCHAR     connIn[MAX_NAME_LEN*2], connOut[MAX_NAME_LEN*2];
+  /* Buffers have to be large enough to contain SSL options and long names */
+  SQLCHAR     connIn[4096], connOut[4096];
+  SQLCHAR     dsn_buf[MAX_NAME_LEN]= {0}, socket_buf[MAX_NAME_LEN]= {0};
+  SQLCHAR     db_buf[MAX_NAME_LEN]= {0}, port_buf[MAX_NAME_LEN]= {0};
   SQLSMALLINT len;
   SQLRETURN   rc;
 
+  /* We never set the custom DSN, but sometimes use DRIVER instead */
+  if (dsn == NULL)
+    sprintf((char *)dsn_buf, "DSN=%s", (char *)mydsn);
+  else if (dsn == USE_DRIVER)
+    sprintf((char *)dsn_buf, "DRIVER=%s", (char *)mydriver);
+  else
+    sprintf((char *)dsn_buf, "DSN=%s", (char *)dsn);
+
   /* Defaults */
-  if (dsn     == NULL) dsn=     mydsn;
   if (uid     == NULL) uid=     myuid;
   if (pwd     == NULL) pwd=     mypwd;
+  if (db      == NULL) db=      mydb;
   if (options == NULL) options= my_str_options;
 
-  sprintf((char *)connIn, "DSN=%s;UID=%s;PWD=%s;DATABASE=test;OPTION=%d",
-          (char *)dsn, (char *)uid, (char *)pwd, myoption);
+  sprintf((char *)connIn, "%s;UID=%s;PWD=%s;OPTION=%d",
+          (char *)dsn_buf, (char *)uid, (char *)pwd, myoption);
 
   if (mysock && mysock[0])
   {
-    strcat((char *)connIn, ";SOCKET=");
-    strcat((char *)connIn, (char *)mysock);
+    sprintf((char *)socket_buf, ";SOCKET=%s", (char *)mysock);
+    strcat((char *)connIn, socket_buf);
+  }
+  if (db && db[0])
+  {
+    sprintf((char *)db_buf, ";DATABASE=%s", (char *)db);
+    strcat((char *)connIn, (char *)db_buf);
   }
   if (myport)
   {
-    char buff[20];
-    sprintf(buff, ";PORT=%d", myport);
-    strcat((char *)connIn, buff);
+    sprintf(port_buf, ";PORT=%d", myport);
+    strcat((char *)connIn, port_buf);
   }
 
   if (options != NULL && options[0] > 0)
@@ -1129,7 +1194,7 @@ int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
   }
 
   rc= SQLDriverConnect(*hdbc, NULL, connIn, SQL_NTS, connOut,
-                                 MAX_NAME_LEN, &len, SQL_DRIVER_NOPROMPT);
+                       MAX_NAME_LEN, &len, SQL_DRIVER_NOPROMPT);
 
   if (SQL_SUCCEEDED(rc))
   {
@@ -1138,11 +1203,22 @@ int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
     rc= SQLSetConnectAttr(*hdbc, SQL_ATTR_AUTOCOMMIT,
                                   (SQLPOINTER)SQL_AUTOCOMMIT_ON, 0);
   }
+  else
+  {
+    /* re-build and print the connection string with hidden password */
+    printf("# Connection failed with the following Connection string: " \
+           "\n%s;UID=%s;PWD=*******%s%s%s;%s\n", 
+           dsn_buf, uid, socket_buf, db_buf, port_buf, options);
+  }
 
   return rc;
 }
 
-int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
+
+int alloc_basic_handles_with_opt(SQLHENV *henv, SQLHDBC *hdbc, 
+                                 SQLHSTMT *hstmt,  const SQLCHAR *dsn, 
+                                 const SQLCHAR *uid, const SQLCHAR *pwd, 
+                                 const SQLCHAR *db, const SQLCHAR *options)
 {
   ok_env(*henv, SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, henv));
 
@@ -1151,7 +1227,7 @@ int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
 
   ok_env(*henv, SQLAllocHandle(SQL_HANDLE_DBC, *henv, hdbc));
 
-  ok_con(*hdbc, get_connection(hdbc, mydsn, myuid, mypwd, my_str_options));
+  ok_con(*hdbc, get_connection(hdbc, dsn, uid, pwd, db, options));
 
   ok_con(*hdbc, SQLAllocHandle(SQL_HANDLE_STMT, *hdbc, hstmt));
 
@@ -1159,6 +1235,15 @@ int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
 }
 
 
+int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
+{
+  return alloc_basic_handles_with_opt(henv, hdbc, hstmt, (SQLCHAR *)mydsn, 
+                                      (SQLCHAR *)myuid, (SQLCHAR *)mypwd,
+                                      (SQLCHAR *)mydb, 
+                                      (SQLCHAR *)my_str_options);
+}
+
+
 int free_basic_handles(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt)
 {
   /* We don't care if this succeeds, the connection may have gone away. */
