@@ -164,6 +164,8 @@ static SQLWCHAR W_NO_I_S[]= {'N','O','_','I','_','S',0};
 static SQLWCHAR W_PREFETCH[]= {'P','R','E','F','E','T','C','H',0};
 static SQLWCHAR W_CAN_HANDLE_EXP_PWD[]=
   {'C','A','N','_','H','A','N','D','L','E','_','E','X','P','_','P','W','D',0};
+static SQLWCHAR W_ENABLE_CLEARTEXT_PLUGIN[]=
+  {'E','N','A','B','L','E','_','C','L','E','A', 'R', 'T', 'E', 'X', 'T', '_','P','L','U','G','I','N',0};
 
 /* DS_PARAM */
 /* externally used strings */
@@ -192,7 +194,8 @@ SQLWCHAR *dsnparams[]= {W_DSN, W_DRIVER, W_DESCRIPTION, W_SERVER,
                         W_ZERO_DATE_TO_MIN, W_MIN_DATE_TO_ZERO,
                         W_MULTI_STATEMENTS, W_COLUMN_SIZE_S32,
                         W_NO_BINARY_RESULT, W_DFLT_BIGINT_BIND_STR,
-                        W_CLIENT_INTERACTIVE, W_NO_I_S, W_PREFETCH, W_CAN_HANDLE_EXP_PWD};
+                        W_CLIENT_INTERACTIVE, W_NO_I_S, W_PREFETCH, 
+                        W_CAN_HANDLE_EXP_PWD, W_ENABLE_CLEARTEXT_PLUGIN};
 static const
 int dsnparamcnt= sizeof(dsnparams) / sizeof(SQLWCHAR *);
 /* DS_PARAM */
@@ -809,6 +812,8 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *booldest= &ds->no_information_schema;
   else if (!sqlwcharcasecmp(W_CAN_HANDLE_EXP_PWD, param))
     *booldest= &ds->can_handle_exp_pwd;
+  else if (!sqlwcharcasecmp(W_ENABLE_CLEARTEXT_PLUGIN, param))
+    *booldest= &ds->enable_cleartext_plugin;
 
   /* DS_PARAM */
 }
@@ -1009,7 +1014,9 @@ int ds_from_kvpair(DataSource *ds, const SQLWCHAR *attrs, SQLWCHAR delim)
 
     attrs= end;
     /* If delim is NULL then double-NULL is the end of key-value pairs list */
-    while ((delim && *attrs == delim) || *attrs == ' ')
+    while ((delim && *attrs == delim) ||
+           (!delim && !*attrs && *(attrs+1)) ||
+           *attrs == ' ')
       ++attrs;
   }
 
@@ -1270,6 +1277,7 @@ int ds_add(DataSource *ds)
   if (ds_add_intprop(ds->name, W_DFLT_BIGINT_BIND_STR, ds->default_bigint_bind_str)) goto error;
   if (ds_add_intprop(ds->name, W_NO_I_S, ds->no_information_schema)) goto error;
   if (ds_add_intprop(ds->name, W_CAN_HANDLE_EXP_PWD, ds->can_handle_exp_pwd)) goto error;
+  if (ds_add_intprop(ds->name, W_ENABLE_CLEARTEXT_PLUGIN, ds->enable_cleartext_plugin)) goto error;
   /* DS_PARAM */
 
   rc= 0;
